#include <stdio.h>
#include <stdlib.h>
#include <linux/input.h>
#include <linux/netlink.h>
#include <linux/rtnetlink.h>
#include <linux/if_addr.h>
#include <sys/socket.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#define IFACE_ID 3
#define DATA_LENGTH 32768

struct packet {
    struct nlmsghdr hdr;
    struct ifaddrmsg msg;
};


void generate_address_request(struct packet* pack, int interface_id) {
    memset(pack, 0, sizeof(struct packet));
    pack->hdr.nlmsg_len = sizeof(struct packet);
    pack->hdr.nlmsg_type = RTM_GETADDR;
    pack->hdr.nlmsg_flags = NLM_F_REQUEST;
    pack->msg.ifa_index = interface_id;  // This gets ignored, why?
}

void parse_all_packets(char* messages_bytes, int total_length) {
    int index = 0, cursor = 0;
    while (cursor < total_length) {
        struct nlmsghdr* header = (struct nlmsghdr*) (messages_bytes + cursor);
        if (header->nlmsg_type == NLMSG_ERROR) {
            int* errnumber = (int*) (messages_bytes + cursor + sizeof(struct nlmsghdr));
            printf("Couldn't parse message #%d, errno: %d!\n", index, *errnumber);
        } else if (header->nlmsg_type == NLMSG_DONE) {
            printf("Message #%d, was the last one!\n", index);
            continue;
        } else {
            struct ifaddrmsg* message = (struct ifaddrmsg*) (messages_bytes + cursor + sizeof(struct nlmsghdr));
            printf("Message #%d contains interface #%d data, e.g. prefixlen: %d!\n", index, message->ifa_index, message->ifa_prefixlen);
        }
        cursor += header->nlmsg_len;
        index++;
    }
}

int main() {
    struct sockaddr_nl addr;
    memset(&addr, 0, sizeof(struct sockaddr_nl));
    addr.nl_family = AF_NETLINK;

    int sock = socket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
    if (sock == -1) {
        printf("Couldn't create netlink socket, errno: %d!\n", errno);
        exit(1);
    }

    int err = bind(sock, (struct sockaddr *) &addr, sizeof(struct sockaddr_nl));
    if (err != 0) {
        printf("Couldn't bind netlink socket, errno: %d!\n", errno);
        exit(1);
    }

    socklen_t len =  sizeof(struct sockaddr_nl);
    getsockname(sock, (struct sockaddr *) &addr, &len);

    struct packet pack;
    generate_address_request(&pack, IFACE_ID);
    write(sock, &pack, sizeof(struct packet));

    char buf[DATA_LENGTH];
    memset(buf, 0, DATA_LENGTH);
    int response_len = read(sock, buf, (socklen_t) DATA_LENGTH);
    parse_all_packets(buf, response_len);
}
